<?php
#BEGIN_LICENSE
#-------------------------------------------------------------------------
# Module: Promotions (c) 2008 by Robert Campbell 
#         (calguy1000@cmsmadesimple.org)
#  An addon module for CMS Made Simple to allow creating, interfacing with
#  and managing promotions or sales in the E-comemrce suite.
# 
#-------------------------------------------------------------------------
# CMS - CMS Made Simple is (c) 2005 by Ted Kulp (wishy@cmsmadesimple.org)
# This project's homepage is: http://www.cmsmadesimple.org
#
#-------------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# However, as a special exception to the GPL, this software is distributed
# as an addon module to CMS Made Simple.  You may not use this software
# in any Non GPL version of CMS Made simple, or in any version of CMS
# Made simple that does not indicate clearly and obviously in its admin 
# section that the site was built with CMS Made simple.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# Or read it online: http://www.gnu.org/licenses/licenses.html#GPL
#
#-------------------------------------------------------------------------
#END_LICENSE

require_once(dirname(__FILE__).'/class.promo_ops.php');

define('PROMOTIONS_COND_FEU','PROMOTIONS_COND_FEU');       // FEU group membership
define('PROMOTIONS_COND_SUBTOTAL','PROMOTIONS_COND_SUBTOTAL');   // Order total (pre shipping and taxes)
define('PROMOTIONS_COND_WEIGHT','PROMOTIONS_COND_WEIGHT'); // Order weight 
define('PROMOTIONS_COND_PRODID','PROMOTIONS_COND_PRODID'); // Order has a certain product id
define('PROMOTIONS_COND_PRODCAT','PROMOTIONS_COND_PRODCAT'); // Order has a product from a certain category
define('PROMOTIONS_COND_PRODHIER','PROMOTIONS_COND_PRODHIER'); // Order has a product from a certain hierarchy
define('PROMOTIONS_COND_COUPON','PROMOTIONS_COND_COUPON'); // Order has a product from a certain hierarchy
define('PROMOTIONS_OFFER_PRODUCT','01_PROMOTIONS_OFFER_PRODUCT');
define('PROMOTIONS_OFFER_PRODDISCOUNT','02_PROMOTIONS_OFFER_PRODDISCOUNT');
define('PROMOTIONS_OFFER_PERCENT','03_PROMOTIONS_OFFER_PERCENT');
define('PROMOTIONS_OFFER_DISCOUNT','04_PROMOTIONS_OFFER_DISCOUNT');

class promotion_condition
{
  private $_id;
  private $_promotion_id;
  private $_cond_type;
  private $_data;

  public static function get_types_array()
  {
    return array(PROMOTIONS_COND_FEU,PROMOTIONS_COND_SUBTOTAL,PROMOTIONS_COND_PRODID,
		 PROMOTIONS_COND_PRODCAT,PROMOTIONS_COND_PRODHIER,
		 PROMOTIONS_COND_COUPON,PROMOTIONS_COND_WEIGHT);
  }

  public function get_id()
  {
    return $this->_id;
  }

  public function set_id($id)
  {
    $this->_id = $id;
  }

  public function get_promotion_id()
  {
    return $this->_promotion_id;
  }

  public function set_promotion_id($id)
  {
    $this->_promotion_id = $id;
  }

  public function get_cond_type()
  {
    return $this->_cond_type;
  }

  public function set_cond_type($type)
  {
    if( $type == PROMOTIONS_COND_FEU ||
	$type == PROMOTIONS_COND_SUBTOTAL ||
	$type == PROMOTIONS_COND_WEIGHT ||
	$type == PROMOTIONS_COND_COUPON ||
	$type == PROMOTIONS_COND_PRODID ||
	$type == PROMOTIONS_COND_PRODCAT ||
	$type == PROMOTIONS_COND_PRODHIER )
      $this->_cond_type = $type;
  }

  public function get_data()
  {
    return $this->_data;
  }

  public function set_data($data)
  {
    $this->_data = $data;
  }
} // class


class promotion
{
  private $_id;
  private $_name;
  private $_description;
  private $_image;
  private $_created;
  private $_start_date;
  private $_end_date;
  private $_offer_type;
  private $_offer_data;
  private $_conditions;

  static public function get_offer_array()
  {
    return array(PROMOTIONS_OFFER_PERCENT,PROMOTIONS_OFFER_DISCOUNT,PROMOTIONS_OFFER_PRODUCT,
		 PROMOTIONS_OFFER_PRODDISCOUNT);
  }

  public function get_id()
  {
    return $this->_id;
  }

  public function set_id($val)
  {
    $this->_id = $val;
  }

  public function get_name()
  {
    return $this->_name;
  }

  public function set_name($val)
  {
    $this->_name = $val;
  }

  public function get_description()
  {
    return $this->_description;
  }

  public function set_description($val)
  {
    $this->_description = $val;
  }

  public function get_image()
  {
    return $this->_image;
  }

  public function set_image($val)
  {
    if( empty($val) || (is_int($val) && ($val < 1)) ) 
      {
	return FALSE;
      }
    $this->_image = $val;
  }

  public function get_created()
  {
    return $this->_created;
  }

  public function set_created($val)
  {
    $this->_created = $val;
  }

  public function get_start_date()
  {
    return $this->_start_date;
  }


  public function set_start_date($val)
  {
    $this->_start_date = $val;
  }


  public function get_end_date()
  {
    return $this->_end_date;
  }


  public function set_end_date($val)
  {
    $this->_end_date = $val;
  }


  public function get_offer_type()
  {
    return $this->_offer_type;
  }


  public function set_offer_type($val)
  {
    $this->_offer_type = $val;
  }


  public function get_offer_data()
  {
    return $this->_offer_data;
  }

  public function set_offer_data($data)
  {
    $this->_offer_data = $data;
  }

  public function get_conditions()
  {
    return $this->_conditions;
  }

  public function add_condition(&$condition)
  {
    if( !is_a($condition,'promotion_condition') ) 
      return FALSE;

    if( !is_array($this->_conditions) )
      {
	$this->_conditions = array();
      }
    $this->_conditions[] = $condition;
  }

  public function del_condition($idx)
  {
    if( is_array($this->_conditions) &&
	count($this->_conditions) > $idx &&
	$idx > 0)
      {
	$tmp = array();
	for( $i = 0; $i < count($this->_conditions); $i++ )
	  {
	    if( $i == $idx ) continue;
	    $tmp[] = $this->_conditions;
	  }
	$this->_conditions = $tmp;
      }
  }

  public function del_conditions()
  {
    $this->_conditions = null;
  }

  public function &get_condition($idx)
  {
    if( !is_array($this->_conditions) ||
	(count($this->_conditions) < $idx) ||
	($idx < 0) )
      {
	$tmp = FALSE;
	return $tmp;
      }

    return $this->_conditions[$idx];
  }

  public function count_conditions()
  {
    return count($this->_conditions);
  }


  public function save()
  {
    if( $this->_id )
      {
	return $this->_update();
      }
    return $this->_insert();
  }


  private function _update()
  {
    return promo_ops::update_promo($this);
  }


  private function _insert()
  {
    return promo_ops::insert_promo($this);
  }


  public function check_valid()
  {
    $st = cge_utils::unix_time($this->get_start_date());
    $et = cge_utils::unix_time($this->get_end_date());
    $cr = cge_utils::unix_time($this->get_created());

    if( $st < $cr ) return FALSE;
    if( $et <= $st ) return FALSE;
    if( empty($this->_name) ) return FALSE;
    if( empty($this->_offer_data) ) return FALSE;

    return TRUE;
  }
} // class

#
# EOF
#
?>